@@ -2,7 +2,7 @@
 
 #' `print()` and `str()` generics.
 #'
-#' These are contructed to be more easily extensible since you can override
+#' These are constructed to be more easily extensible since you can override
 #' the `_header()`, `_data()` or `_footer()` components individually. The
 #' default methods are built on top of `format()`.
 #'
@@ -39,7 +39,7 @@ obj_print_data <- function(x, ...) {
 #' @export
 obj_print_data.default <- function(x, ...) {
   if (length(x) == 0)
-    return()
+    return(invisible(x))
 
   out <- stats::setNames(format(x), names(x))
   print(out, quote = FALSE)
@@ -163,4 +163,3 @@ obj_str_footer.default <- function(x, ...,
 
   invisible(x)
 }
-
