@@ -28,13 +28,17 @@
 #'   length. This means that `vec_unrep()` works on data frames by compressing
 #'   repeated rows.
 #'
+#' @inheritParams rlang::args_error_context
+#' @inheritParams rlang::args_dots_empty
 #' @param x A vector.
 #' @param times
 #'   For `vec_rep()`, a single integer for the number of times to repeat
 #'   the entire vector.
 #'
 #'   For `vec_rep_each()`, an integer vector of the number of times to repeat
-#'   each element of `x`. `times` will be recycled to the size of `x`.
+#'   each element of `x`. `times` will be [recycled][vector_recycling_rules] to
+#'   the size of `x`.
+#' @param x_arg,times_arg Argument names for errors.
 #'
 #' @return
 #' For `vec_rep()`, a vector the same type as `x` with size
@@ -82,18 +86,30 @@ NULL
 
 #' @rdname vec-rep
 #' @export
-vec_rep <- function(x, times) {
-  .Call(vctrs_rep, x, times)
+vec_rep <- function(x,
+                    times,
+                    ...,
+                    error_call = current_env(),
+                    x_arg = "x",
+                    times_arg = "times") {
+  check_dots_empty0(...)
+  .Call(ffi_vec_rep, x, times, environment())
 }
 
 #' @rdname vec-rep
 #' @export
-vec_rep_each <- function(x, times) {
-  .Call(vctrs_rep_each, x, times)
+vec_rep_each <- function(x,
+                         times,
+                         ...,
+                         error_call = current_env(),
+                         x_arg = "x",
+                         times_arg = "times") {
+  check_dots_empty0(...)
+  .Call(ffi_vec_rep_each, x, times, environment())
 }
 
 #' @rdname vec-rep
 #' @export
 vec_unrep <- function(x) {
-  .Call(vctrs_unrep, x)
+  .Call(ffi_vec_unrep, x, environment())
 }
