@@ -390,6 +390,15 @@ test_that("df-cols aren't flattened, so `condition` is applied jointly on the df
   expect_identical(res$haystack, 1L)
 })
 
+test_that("must have at least 1 column to match", {
+  expect_snapshot(error = TRUE, {
+    vec_locate_matches(data_frame(), data_frame())
+  })
+  expect_snapshot(error = TRUE, {
+    vec_locate_matches(data_frame(), data_frame(), error_call = call("foo"))
+  })
+})
+
 # ------------------------------------------------------------------------------
 # vec_locate_matches() - rcrd
 
@@ -441,6 +450,104 @@ test_that("rcrd type incompleteness is handled correctly", {
   expect_identical(res$haystack, c(NA_integer_, NA_integer_))
 })
 
+# ------------------------------------------------------------------------------
+# vec_locate_matches() - S3
+
+test_that("S3 types with order proxies that depend on the data are combined before the proxy is taken", {
+  # i.e. `bignum:::vec_proxy_order.bignum_biginteger()`
+
+  x <- structure(c(5L, 1L), class = "foo")
+  y <- structure(c(8L, 5L), class = "foo")
+
+  local_methods(
+    vec_proxy_order.foo = function(x, ...) {
+      rank(unclass(x))
+    }
+  )
+
+  # Can't take the order proxies separately because they are the same!
+  expect_identical(vec_proxy_order(x), vec_proxy_order(y))
+
+  res <- vec_locate_matches(x, y)
+  expect_identical(res$needles, c(1L, 2L))
+  expect_identical(res$haystack, c(2L, NA))
+
+  x_df <- data_frame(a = x, b = x)
+  y_df <- data_frame(a = y, b = y)
+
+  res <- vec_locate_matches(x_df, y_df)
+  expect_identical(res$needles, c(1L, 2L))
+  expect_identical(res$haystack, c(2L, NA))
+})
+
+test_that("Works with base R S3 types we support natively", {
+  x <- new_factor(c(1L, 2L), levels = c("x", "y"))
+  y <- new_factor(c(3L, 1L, 1L), levels = c("x", "y", "z"))
+  res <- vec_locate_matches(x, y)
+  expect_identical(res$needles, c(1L, 1L, 2L))
+  expect_identical(res$haystack, c(2L, 3L, NA))
+
+  x <- new_ordered(c(1L, 2L), levels = c("x", "y"))
+  y <- new_ordered(c(2L, 1L, 1L), levels = c("x", "y"))
+  res <- vec_locate_matches(x, y)
+  expect_identical(res$needles, c(1L, 1L, 2L))
+  expect_identical(res$haystack, c(2L, 3L, 1L))
+
+  x <- new_date(c(1, 2))
+  y <- new_date(c(3, 1, 1))
+  res <- vec_locate_matches(x, y)
+  expect_identical(res$needles, c(1L, 1L, 2L))
+  expect_identical(res$haystack, c(2L, 3L, NA))
+
+  x <- new_datetime(c(1, 2))
+  y <- new_datetime(c(3, 1, 1))
+  res <- vec_locate_matches(x, y)
+  expect_identical(res$needles, c(1L, 1L, 2L))
+  expect_identical(res$haystack, c(2L, 3L, NA))
+
+  x <- as.POSIXlt(new_datetime(c(1, 2)))
+  y <- as.POSIXlt(new_datetime(c(3, 1, 1)))
+  res <- vec_locate_matches(x, y)
+  expect_identical(res$needles, c(1L, 1L, 2L))
+  expect_identical(res$haystack, c(2L, 3L, NA))
+})
+
+test_that("Works with classed data frame columns", {
+  x_col <- new_data_frame(list(a = c(1L, 2L), b = c(2, 3)), class = "foo")
+  y_col <- new_data_frame(list(a = c(1L, 1L, 1L), b = c(2, 4, 2)), class = "foo")
+
+  x <- new_data_frame(list(c = c(1L, 1L), d = x_col))
+  y <- new_data_frame(list(c = c(1L, 1L, 1L), d = y_col))
+
+  res <- vec_locate_matches(x, y)
+  expect_identical(res$needles, c(1L, 1L, 2L))
+  expect_identical(res$haystack, c(1L, 3L, NA))
+})
+
+test_that("AsIs types are combined before order proxies are taken (#1557)", {
+  x <- I(list(5, 1))
+  y <- I(list(8, 5, 5))
+
+  res <- vec_locate_matches(x, y)
+  expect_identical(res$needles, c(1L, 1L, 2L))
+  expect_identical(res$haystack, c(2L, 3L, NA))
+})
+
+# ------------------------------------------------------------------------------
+# vec_locate_matches() - ptype2 / casting
+
+test_that("common type of `needles` and `haystack` is taken", {
+  x <- 1
+  y <- "a"
+
+  expect_snapshot(error = TRUE, {
+    vec_locate_matches(x, y)
+  })
+  expect_snapshot(error = TRUE, {
+    vec_locate_matches(x, y, needles_arg = "x", error_call = call("foo"))
+  })
+})
+
 # ------------------------------------------------------------------------------
 # vec_locate_matches() - missing values
 
@@ -710,6 +817,7 @@ test_that("`incomplete` can error informatively", {
   expect_snapshot({
     (expect_error(vec_locate_matches(NA, 1, incomplete = "error")))
     (expect_error(vec_locate_matches(NA, 1, incomplete = "error", needles_arg = "foo")))
+    (expect_error(vec_locate_matches(NA, 1, incomplete = "error", needles_arg = "foo", error_call = call("fn"))))
   })
 })
 
@@ -722,6 +830,8 @@ test_that("`incomplete` is validated", {
     (expect_error(vec_locate_matches(1, 2, incomplete = 1.5)))
     (expect_error(vec_locate_matches(1, 2, incomplete = c("match", "drop"))))
     (expect_error(vec_locate_matches(1, 2, incomplete = "x")))
+    # Uses internal call
+    (expect_error(vec_locate_matches(1, 2, incomplete = "x", error_call = call("fn"))))
   })
 })
 
@@ -795,69 +905,6 @@ test_that("duplicate needles match the same haystack locations", {
   expect_identical(x$haystack, c(1L, 3L, 2L, 1L, 3L, 2L))
 })
 
-test_that("`multiple` can error informatively", {
-  expect_snapshot({
-    (expect_error(vec_locate_matches(1L, c(1L, 1L), multiple = "error")))
-    (expect_error(vec_locate_matches(1L, c(1L, 1L), multiple = "error", needles_arg = "foo")))
-    (expect_error(vec_locate_matches(1L, c(1L, 1L), multiple = "error", needles_arg = "foo", haystack_arg = "bar")))
-  })
-})
-
-test_that("`multiple` can warn informatively", {
-  expect_snapshot({
-    (expect_warning(vec_locate_matches(1L, c(1L, 1L), multiple = "warning")))
-    (expect_warning(vec_locate_matches(1L, c(1L, 1L), multiple = "warning", needles_arg = "foo")))
-    (expect_warning(vec_locate_matches(1L, c(1L, 1L), multiple = "warning", needles_arg = "foo", haystack_arg = "bar")))
-  })
-})
-
-test_that("warning falls back to 'all'", {
-  expect_warning(
-    result <- vec_locate_matches(c(1L, 3L, 1L, 3L), c(1L, 3L, 1L), multiple = "warning"),
-    class = "vctrs_warning_matches_multiple"
-  )
-
-  expect_identical(
-    result,
-    vec_locate_matches(c(1L, 3L, 1L, 3L), c(1L, 3L, 1L), multiple = "all")
-  )
-})
-
-test_that("errors on multiple matches that come from different nesting containers", {
-  df <- data_frame(x = 0, y = 0)
-  df2 <- data_frame(x = 1:2, y = 2:1)
-
-  expect_error(
-    vec_locate_matches(df, df2, condition = c("<=", "<="), multiple = "error"),
-    "multiple matches"
-  )
-})
-
-test_that("errors when a match from a different nesting container is processed early on", {
-  # Row 1 has 2 matches
-  # Row 2 has 0 matches
-  needles <- data_frame(
-    a = c(1, 8),
-    b = c(2, 9)
-  )
-
-  # Rows 1 and 2 end up in different nesting containers
-  haystack <- data_frame(
-    a = c(5, 6),
-    b = c(7, 6)
-  )
-
-  # needles[1,] records the haystack[1,] match first, which is in the 1st
-  # value of `loc_first_match_o_haystack`, then records the haystack[3,] match
-  # which is in the 3rd value of `loc_first_match_o_haystack` even though it
-  # is processed 2nd (i.e. we need to use `loc` rather than `i` when detecting
-  # multiple matches)
-  expect_error(
-    vec_locate_matches(needles, haystack, condition = "<", multiple = "error"),
-    "multiple matches"
-  )
-})
-
 test_that("correctly gets all matches when they come from different nesting containers", {
   needles <- data_frame(
     a = c(1, 8),
@@ -910,16 +957,386 @@ test_that("correctly gets first/last/any match when they come from different nes
   )
 })
 
+test_that("`multiple` is validated", {
+  expect_snapshot({
+    (expect_error(vec_locate_matches(1, 2, multiple = 1.5)))
+    (expect_error(vec_locate_matches(1, 2, multiple = c("first", "last"))))
+    (expect_error(vec_locate_matches(1, 2, multiple = "x")))
+    # Uses internal error
+    (expect_error(vec_locate_matches(1, 2, multiple = "x", error_call = call("fn"))))
+  })
+})
+
+# ------------------------------------------------------------------------------
+# vec_locate_matches() - `multiple` (deprecated)
+
+test_that("`multiple` can error informatively", {
+  expect_snapshot({
+    (expect_error(vec_locate_matches(1L, c(1L, 1L), multiple = "error")))
+    (expect_error(vec_locate_matches(1L, c(1L, 1L), multiple = "error", needles_arg = "foo")))
+    (expect_error(vec_locate_matches(1L, c(1L, 1L), multiple = "error", needles_arg = "foo", error_call = call("fn"))))
+    (expect_error(vec_locate_matches(1L, c(1L, 1L), multiple = "error", needles_arg = "foo", haystack_arg = "bar")))
+  })
+})
+
+test_that("`multiple` can warn informatively", {
+  expect_snapshot({
+    (expect_warning(vec_locate_matches(1L, c(1L, 1L), multiple = "warning")))
+    (expect_warning(vec_locate_matches(1L, c(1L, 1L), multiple = "warning", needles_arg = "foo")))
+    (expect_warning(vec_locate_matches(1L, c(1L, 1L), multiple = "warning", needles_arg = "foo", error_call = call("fn"))))
+    (expect_warning(vec_locate_matches(1L, c(1L, 1L), multiple = "warning", needles_arg = "foo", haystack_arg = "bar")))
+  })
+})
+
+test_that("warning falls back to 'all'", {
+  expect_warning(
+    result <- vec_locate_matches(c(1L, 3L, 1L, 3L), c(1L, 3L, 1L), multiple = "warning"),
+    class = "vctrs_warning_matches_multiple"
+  )
+
+  expect_identical(
+    result,
+    vec_locate_matches(c(1L, 3L, 1L, 3L), c(1L, 3L, 1L), multiple = "all")
+  )
+})
+
+test_that("errors on multiple matches that come from different nesting containers", {
+  df <- data_frame(x = 0, y = 0)
+  df2 <- data_frame(x = 1:2, y = 2:1)
+
+  expect_snapshot(error = TRUE, {
+    vec_locate_matches(df, df2, condition = c("<=", "<="), multiple = "error")
+  })
+})
+
+test_that("errors when a match from a different nesting container is processed early on", {
+  # Row 1 has 2 matches
+  # Row 2 has 0 matches
+  needles <- data_frame(
+    a = c(1, 8),
+    b = c(2, 9)
+  )
+
+  # Rows 1 and 2 end up in different nesting containers
+  haystack <- data_frame(
+    a = c(5, 6),
+    b = c(7, 6)
+  )
+
+  # needles[1,] records the haystack[1,] match first, which is in the 1st
+  # value of `loc_first_match_o_haystack`, then records the haystack[3,] match
+  # which is in the 3rd value of `loc_first_match_o_haystack` even though it
+  # is processed 2nd (i.e. we need to use `loc` rather than `i` when detecting
+  # multiple matches)
+  expect_snapshot(error = TRUE, {
+    vec_locate_matches(needles, haystack, condition = "<", multiple = "error")
+  })
+})
+
 test_that("`multiple = 'error'` doesn't error errneously on the last observation", {
   expect_error(res <- vec_locate_matches(1:2, 1:2, multiple = "error"), NA)
   expect_identical(res$needles, 1:2)
   expect_identical(res$haystack, 1:2)
 })
 
-test_that("`multiple` is validated", {
-  expect_error(vec_locate_matches(1, 2, multiple = 1.5), "`multiple` must be a string")
-  expect_error(vec_locate_matches(1, 2, multiple = c("first", "last")), "`multiple` must be a string")
-  expect_error(vec_locate_matches(1, 2, multiple = "x"), '`multiple` must be one of "all", "any", "first", "last", "warning", or "error"')
+test_that("`multiple = 'error' / 'warning'` throw correctly when combined with `relationship`", {
+  x <- c(1, 2, 2)
+  y <- c(2, 1, 2)
+
+  # `multiple` error technically fires first
+  expect_snapshot({
+    (expect_error(vec_locate_matches(x, y, relationship = "one-to-one", multiple = "error")))
+  })
+
+  # Works when warning is also requested
+  expect_snapshot({
+    (expect_error(vec_locate_matches(x, y, relationship = "warn-many-to-many", multiple = "error")))
+  })
+  # Both warnings are thrown if applicable
+  expect_snapshot({
+    vec_locate_matches(x, y, relationship = "warn-many-to-many", multiple = "warning")
+  })
+  # Both warning and error are thrown if applicable
+  expect_snapshot(error = TRUE, {
+    vec_locate_matches(x, y, relationship = "one-to-one", multiple = "warning")
+  })
+
+  x <- c(1, 2)
+  y <- c(2, 1, 2)
+
+  expect_snapshot({
+    (expect_error(vec_locate_matches(x, y, relationship = "warn-many-to-many", multiple = "error")))
+  })
+  # Only `multiple` warning is applicable here
+  expect_snapshot({
+    vec_locate_matches(x, y, relationship = "warn-many-to-many", multiple = "warning")
+  })
+})
+
+# ------------------------------------------------------------------------------
+# vec_locate_matches() - `relationship`
+
+test_that("`relationship` handles one-to-one case", {
+  # No error
+  expect_identical(
+    vec_locate_matches(1:2, 2:1, relationship = "one-to-one"),
+    vec_locate_matches(1:2, 2:1)
+  )
+
+  # Doesn't care about the zero match case
+  expect_identical(
+    vec_locate_matches(1:2, 3:4, relationship = "one-to-one"),
+    vec_locate_matches(1:2, 3:4)
+  )
+
+  expect_snapshot({
+    (expect_error(vec_locate_matches(c(2, 1), c(1, 1), relationship = "one-to-one")))
+    (expect_error(vec_locate_matches(c(1, 1), c(1, 2), relationship = "one-to-one")))
+  })
+})
+
+test_that("`relationship` handles one-to-many case", {
+  # No error
+  expect_identical(
+    vec_locate_matches(c(1, 2), c(1, 2, 2), relationship = "one-to-many"),
+    vec_locate_matches(c(1, 2), c(1, 2, 2))
+  )
+
+  # Doesn't care about the zero match case
+  expect_identical(
+    vec_locate_matches(1:2, 3:4, relationship = "one-to-many"),
+    vec_locate_matches(1:2, 3:4)
+  )
+
+  expect_snapshot({
+    (expect_error(vec_locate_matches(c(1, 2, 2), c(2, 1), relationship = "one-to-many")))
+  })
+})
+
+test_that("`relationship` handles many-to-one case", {
+  # No error
+  expect_identical(
+    vec_locate_matches(c(1, 2, 2), c(1, 2), relationship = "many-to-one"),
+    vec_locate_matches(c(1, 2, 2), c(1, 2))
+  )
+
+  # Doesn't care about the zero match case
+  expect_identical(
+    vec_locate_matches(1:2, 3:4, relationship = "many-to-one"),
+    vec_locate_matches(1:2, 3:4)
+  )
+
+  expect_snapshot({
+    (expect_error(vec_locate_matches(c(1, 2), c(1, 2, 2), relationship = "many-to-one")))
+  })
+})
+
+test_that("`relationship` handles many-to-many case", {
+  # No error
+  expect_identical(
+    vec_locate_matches(c(1, 2, 2), c(1, 2), relationship = "many-to-many"),
+    vec_locate_matches(c(1, 2, 2), c(1, 2))
+  )
+
+  # No error
+  expect_identical(
+    vec_locate_matches(c(1, 2), c(1, 2, 2), relationship = "many-to-many"),
+    vec_locate_matches(c(1, 2), c(1, 2, 2))
+  )
+
+  # No error
+  expect_identical(
+    vec_locate_matches(c(1, 1, 2), c(1, 2, 2), relationship = "many-to-many"),
+    vec_locate_matches(c(1, 1, 2), c(1, 2, 2))
+  )
+
+  # Doesn't care about the zero match case
+  expect_identical(
+    vec_locate_matches(1:2, 3:4, relationship = "many-to-many"),
+    vec_locate_matches(1:2, 3:4)
+  )
+})
+
+test_that("`relationship` handles warn-many-to-many case", {
+  # No warning
+  expect_identical(
+    expect_silent(
+      vec_locate_matches(c(1, 2, 2), c(1, 2), relationship = "warn-many-to-many")
+    ),
+    vec_locate_matches(c(1, 2, 2), c(1, 2))
+  )
+
+  # No warning
+  expect_identical(
+    expect_silent(
+      vec_locate_matches(c(1, 2), c(1, 2, 2), relationship = "warn-many-to-many")
+    ),
+    vec_locate_matches(c(1, 2), c(1, 2, 2))
+  )
+
+  # Doesn't care about the zero match case
+  expect_identical(
+    expect_silent(
+      vec_locate_matches(1:2, 3:4, relationship = "warn-many-to-many")
+    ),
+    vec_locate_matches(1:2, 3:4)
+  )
+
+  # Specifically designed to ensure we test both:
+  # - Finding multiple `needles` matches before multiple `haystack` matches
+  # - Finding multiple `haystack` matches before multiple `needles` matches
+  expect_snapshot({
+    (expect_warning(vec_locate_matches(c(1, 2, 1), c(1, 2, 2), relationship = "warn-many-to-many")))
+    (expect_warning(vec_locate_matches(c(1, 1, 2), c(2, 2, 1), relationship = "warn-many-to-many")))
+  })
+})
+
+test_that("`relationship` considers `incomplete` matches as possible multiple matches", {
+  x <- c(1, NA, NaN)
+  y <- c(NA, 1)
+
+  expect_snapshot({
+    (expect_error(vec_locate_matches(x, y, relationship = "one-to-many")))
+  })
+
+  # No error
+  expect_identical(
+    vec_locate_matches(x, y, relationship = "one-to-many", incomplete = NA),
+    vec_locate_matches(x, y, incomplete = NA)
+  )
+
+  # No error
+  expect_identical(
+    vec_locate_matches(x, y, relationship = "one-to-many", nan_distinct = TRUE),
+    vec_locate_matches(x, y, nan_distinct = TRUE)
+  )
+})
+
+test_that("`relationship` errors on multiple matches that come from different nesting containers", {
+  df <- data_frame(x = 0, y = 0)
+  df2 <- data_frame(x = 1:2, y = 2:1)
+
+  expect_snapshot({
+    (expect_error(vec_locate_matches(df, df2, condition = c("<=", "<="), relationship = "many-to-one")))
+  })
+})
+
+test_that("`relationship` errors when a match from a different nesting container is processed early on", {
+  # Row 1 has 2 matches
+  # Row 2 has 0 matches
+  needles <- data_frame(
+    a = c(1, 8),
+    b = c(2, 9)
+  )
+
+  # Rows 1 and 2 end up in different nesting containers
+  haystack <- data_frame(
+    a = c(5, 6),
+    b = c(7, 6)
+  )
+
+  # needles[1,] records the haystack[1,] match first, which is in the 1st
+  # value of `loc_first_match_o_haystack`, then records the haystack[3,] match
+  # which is in the 3rd value of `loc_first_match_o_haystack` even though it
+  # is processed 2nd (i.e. we need to use `loc` rather than `i` when detecting
+  # multiple matches)
+  expect_snapshot({
+    (expect_error(vec_locate_matches(needles, haystack, condition = "<", relationship = "many-to-one")))
+  })
+})
+
+test_that("`relationship` doesn't error errneously on the last observation", {
+  expect_error(res <- vec_locate_matches(1:2, 1:2, relationship = "many-to-one"), NA)
+  expect_identical(res$needles, 1:2)
+  expect_identical(res$haystack, 1:2)
+})
+
+test_that("`relationship` doesn't error if `multiple` removes multiple matches", {
+  out <- vec_locate_matches(c(1, 2), c(1, 1), multiple = "any", relationship = "one-to-one")
+  expect_identical(out$needles, c(1L, 2L))
+  expect_identical(out$haystack, c(1L, NA))
+
+  out <- vec_locate_matches(c(1, 2), c(1, 1), multiple = "first", relationship = "one-to-one")
+  expect_identical(out$needles, c(1L, 2L))
+  expect_identical(out$haystack, c(1L, NA))
+
+  out <- vec_locate_matches(c(1, 2), c(1, 1), multiple = "last", relationship = "one-to-one")
+  expect_identical(out$needles, c(1L, 2L))
+  expect_identical(out$haystack, c(2L, NA))
+})
+
+test_that("`relationship` can still detect problematic `haystack` relationships when `multiple = first/last` are used", {
+  expect_snapshot({
+    (expect_error(vec_locate_matches(c(3, 1, 1), c(2, 1, 3, 3), multiple = "first", relationship = "one-to-one")))
+    (expect_error(vec_locate_matches(c(3, 1, 1), c(2, 1, 3, 3), multiple = "first", relationship = "one-to-many")))
+  })
+})
+
+test_that("`relationship` and `remaining` work properly together", {
+  expect_snapshot({
+    out <- vec_locate_matches(
+      c(1, 2, 2),
+      c(2, 3, 1, 1, 4),
+      relationship = "warn-many-to-many",
+      remaining = NA_integer_
+    )
+  })
+  expect_identical(out$needles, c(1L, 1L, 2L, 3L, NA, NA))
+  expect_identical(out$haystack, c(3L, 4L, 1L, 1L, 2L, 5L))
+})
+
+test_that("`relationship` errors if `condition` creates multiple matches", {
+  expect_snapshot({
+    (expect_error(vec_locate_matches(1, c(1, 2), condition = "<=", relationship = "many-to-one")))
+  })
+})
+
+test_that("`relationship` doesn't error if `filter` removes multiple matches", {
+  out <- vec_locate_matches(1, c(1, 2), condition = "<=", filter = "min", relationship = "many-to-one")
+  expect_identical(out$needles, 1L)
+  expect_identical(out$haystack, 1L)
+
+  out <- vec_locate_matches(1, c(1, 2), condition = "<=", filter = "max", relationship = "many-to-one")
+  expect_identical(out$needles, 1L)
+  expect_identical(out$haystack, 2L)
+})
+
+test_that("`relationship` still errors if `filter` hasn't removed all multiple matches", {
+  expect_snapshot({
+    (expect_error(vec_locate_matches(1, c(1, 2, 1), condition = "<=", filter = "min", relationship = "many-to-one")))
+  })
+
+  # But not here
+  out <- vec_locate_matches(c(1, 1), c(1, 2, 1), condition = "<=", filter = "max", relationship = "many-to-one")
+  expect_identical(out$needles, c(1L, 2L))
+  expect_identical(out$haystack, c(2L, 2L))
+})
+
+test_that("`relationship` errors respect argument tags and error call", {
+  expect_snapshot({
+    (expect_error(vec_locate_matches(1L, c(1L, 1L), relationship = "one-to-one", needles_arg = "foo", haystack_arg = "bar", error_call = call("fn"))))
+    (expect_error(vec_locate_matches(c(1L, 1L), 1L, relationship = "one-to-one", needles_arg = "foo", haystack_arg = "bar", error_call = call("fn"))))
+    (expect_error(vec_locate_matches(c(1L, 1L), 1L, relationship = "one-to-many", needles_arg = "foo", haystack_arg = "bar", error_call = call("fn"))))
+    (expect_error(vec_locate_matches(1L, c(1L, 1L), relationship = "many-to-one", needles_arg = "foo", haystack_arg = "bar", error_call = call("fn"))))
+  })
+})
+
+test_that("`relationship` warnings respect argument tags and error call", {
+  expect_snapshot({
+    (expect_warning(vec_locate_matches(c(1L, 1L), c(1L, 1L), relationship = "warn-many-to-many", needles_arg = "foo", haystack_arg = "bar", error_call = call("fn"))))
+    (expect_warning(vec_locate_matches(c(1L, 1L), c(1L, 1L), relationship = "warn-many-to-many", needles_arg = "foo", error_call = call("fn"))))
+    (expect_warning(vec_locate_matches(c(1L, 1L), c(1L, 1L), relationship = "warn-many-to-many", haystack_arg = "bar", error_call = call("fn"))))
+  })
+})
+
+test_that("`relationship` is validated", {
+  expect_snapshot({
+    (expect_error(vec_locate_matches(1, 2, relationship = 1.5)))
+    (expect_error(vec_locate_matches(1, 2, relationship = c("one-to-one", "one-to-many"))))
+    (expect_error(vec_locate_matches(1, 2, relationship = "x")))
+    # Uses internal error
+    (expect_error(vec_locate_matches(1, 2, relationship = "x", error_call = call("fn"))))
+  })
 })
 
 # ------------------------------------------------------------------------------
@@ -964,6 +1381,7 @@ test_that("`no_match` can error informatively", {
   expect_snapshot({
     (expect_error(vec_locate_matches(1, 2, no_match = "error")))
     (expect_error(vec_locate_matches(1, 2, no_match = "error", needles_arg = "foo")))
+    (expect_error(vec_locate_matches(1, 2, no_match = "error", needles_arg = "foo", error_call = call("fn"))))
     (expect_error(vec_locate_matches(1, 2, no_match = "error", needles_arg = "foo", haystack_arg = "bar")))
   })
 })
@@ -997,6 +1415,8 @@ test_that("`no_match` is validated", {
     (expect_error(vec_locate_matches(1, 2, no_match = 1.5)))
     (expect_error(vec_locate_matches(1, 2, no_match = c(1L, 2L))))
     (expect_error(vec_locate_matches(1, 2, no_match = "x")))
+    # Uses internal call
+    (expect_error(vec_locate_matches(1, 2, no_match = "x", error_call = call("fn"))))
   })
 })
 
@@ -1085,6 +1505,7 @@ test_that("`remaining` can error informatively", {
   expect_snapshot({
     (expect_error(vec_locate_matches(1, 2, remaining = "error")))
     (expect_error(vec_locate_matches(1, 2, remaining = "error", needles_arg = "foo")))
+    (expect_error(vec_locate_matches(1, 2, remaining = "error", needles_arg = "foo", error_call = call("fn"))))
     (expect_error(vec_locate_matches(1, 2, remaining = "error", needles_arg = "foo", haystack_arg = "bar")))
   })
 })
@@ -1094,6 +1515,8 @@ test_that("`remaining` is validated", {
     (expect_error(vec_locate_matches(1, 2, remaining = 1.5)))
     (expect_error(vec_locate_matches(1, 2, remaining = c(1L, 2L))))
     (expect_error(vec_locate_matches(1, 2, remaining = "x")))
+    # Uses internal call
+    (expect_error(vec_locate_matches(1, 2, remaining = "x", error_call = call("fn"))))
   })
 })
 
@@ -1315,7 +1738,7 @@ test_that("potential overflow on large output size is caught informatively", {
   # intermediate `r_ssize` will be too large
   skip_if(.Machine$sizeof.pointer < 8L, message = "No long vector support")
 
-  expect_snapshot({
+  expect_snapshot(transform = scrub_internal_error_line_number, {
     (expect_error(vec_locate_matches(1:1e7, 1:1e7, condition = ">=")))
   })
 })
