@@ -1,4 +1,3 @@
-context("test-type-factor")
 
 test_that("ptype methods are descriptive", {
   f <- factor()
@@ -47,12 +46,8 @@ test_that("factor/character coercions are symmetric and unchanging", {
   mat <- maxtype_mat(types)
   expect_true(isSymmetric(mat))
 
-  expect_known_output(
-    mat,
-    print = TRUE,
-    test_path("test-type-factor.txt"),
-    width = 200
-  )
+  local_options(width = 200)
+  expect_snapshot(print(mat))
 })
 
 test_that("factors level are unioned", {
@@ -86,7 +81,7 @@ test_that("vec_ptype2(<factor>, NA) is symmetric (#687)", {
   )
 })
 
-test_that("vec_ptype2(<factor>, NA) is symmetric (#687)", {
+test_that("vec_ptype2(<integer64>, NA) is symmetric (#687)", {
   i64 <- bit64::integer64()
   expect_identical(
     vec_ptype2(i64, NA),
